#-*- coding: UTF-8 -*-

#**********************************************
# R2spec 
#
# Class file for the class Description
#
# Distributed under License GPLv3 or later
# You can find a copy of this license on the website
# http://www.gnu.org/licenses/gpl.html
#
#**********************************************

import os, re, shutil, string

class Description:
    '''
	Description class.
	  Creates and handle the object Description
    '''

    def __init__(self):
        ''' Creates the object '''
        self.license = None
        self.version = None
        self.description = None
        self.summary = None
        self.file = None

    def setVersion(self):
        ''' Set the version of the package '''
        # Retrieve the Version number
        versionMotif = re.compile('Version:(.*)')
        try:
            version = versionMotif.findall(self.file)[0]
            self.version = version.strip()
            try:
                self.version = version.split('-')[0].strip()
            except:
                None
        except:
            self.version = ''	


    def setLicense(self):
        ''' Returns the license of the package '''
        licenseMotif = re.compile('License:(.*)')
        try:
            license = licenseMotif.findall(self.file)[0]
            self.license = license.strip()
        except:
            self.license = ""


    def setDescription(self):
        ''' 
        Returns the description of the package. 
        Catch the description in one or several lines 
        '''
        try:
            description = self.file.split('Description:')[1].split(':')[0].split('\n')
            description = string.join(description[:len(description)-1], "")
            description = description.replace('  ', ' ')
            description = description.replace('\r', '')
            self.description = description.strip()
        except:
            self.description = ""


    def setSummary(self):
        ''' 
        Returns the summary of the package. 
        Catch the title in one or several lines 
        '''
        try:
            summary = self.file.split('Title:')[1].split(':')[0].split('\n')
            summary = string.join(summary[:len(summary)-1], "")
            summary = summary.replace('  ', ' ')
            summary = summary.replace('\r', '')
            summary = summary.strip()
            self.summary = summary[:56]
        except:
            self.summary = ""


    def setRequires(self):
        ''' 
        Returns the required libraries of the package.
        Catch the requirement in one or more lines.
        '''
        # Retrieve the Depends
        try:
            depends = self.file.split('Depends:')[1].split(':')[0].split('\n')
            depends = string.join(depends[:len(depends)-1], "")
            depends = depends.replace('  ', ' ')
            depends = depends.replace('\r', '')
            self.depends = depends.strip()
        except:
            self.depends = ""

        # Retrieve the Suggests
        try:
            suggests = self.file.split('Suggests:')[1].split(':')[0].split('\n')
            suggests = string.join(suggests[:len(suggests)-1], "")
            suggests = suggests.replace('  ', ' ')
            suggests = suggests.replace('\r', '')
            self.suggests = suggests.strip()
        except:
            self.suggests = ""

  
    def setDescriptionFile(self, package):
        ''' Returns the description file of the package '''

        # Save the top directory
        original_directory = os.getcwd()

        # Change directory and load the DESCRIPTION file in memory
        os.chdir("./tmp/"+package.name)
        f = open('DESCRIPTION', 'r')
        self.file = f.read()
        f.close()

        # Comes back to the original directory
        os.chdir(original_directory)