#-*- coding: UTF-8 -*-

#**********************************************
# R2spec 
#
# Class file for the class Package
#
# Distributed under License GPLv3 or later
# You can find a copy of this license on the website
# http://www.gnu.org/licenses/gpl.html
#
#**********************************************

from Packager import *
from Description import *
from Spec import *
from Noarch import *

import os, sys, re, datetime

class Package:
    '''
    Package class.
    It contains all the information about the package that R2spec will create the spec file about.
    '''

    def __init__(self):
        ''' Initiate the object of type Package and set the attributes '''
        self.name = None
        self.source0 = None
        self.URL = None
        self.version = None
        self.date = None
        self.packager = None
        self.description = None
        self.source = None
        self.path = None
        self.created = None
        self.arch = None
    
    def setName(self):
        ''' Get the name of the library from the source file '''
        name = self.source
        name = name.split('_')[0]
        try:
            self.name=name.rsplit('/',1)[1]
        except:
            self.name=name

    def setDate(self):
        ''' Get the current date with the correct format for the spec file '''
        date= datetime.datetime.now().strftime("%a %b %d %Y")
        self.date = date
   
    def setSource(self, source0, source):
        ''' From the url retrieve the name of the source '''
        if source != None and source0 != None:
            if source != source0.rsplit('/',1)[1] :
                sys.exit("Do not give 2 contradictory sources of information")
        elif source != None:
            try:
                self.source = source.rsplit('/',1)[1]
                self.path = source.rsplit('/',1)[0]
            except:
                self.source = source
                self.path = '../'
        elif source0 != None:
            self.source = source0.rsplit('/',1)[1]
            self.path = '../'
            self.source0 = source0
        else:
            sys.exit("The source are not correct ")
    
    def download(self):
        ''' If the package has an url download the source file '''
        if self.source0 != None:
            source=self.source0.rsplit('/',1)[1]
            # Check the presence of the source, download only if they are not already in the working directory
            exist = os.path.isfile(source)
            if exist == True:
                print "\nThe file "+self.source+" already exist, there is no need to download it again."
            else:
                cmd = "wget "+self.source0
                print cmd
                os.system(cmd)
    def openSource(self):
        '''
        Create a tmp directory where the tarball is opened and open
        the tarball.
        '''
        # Creates the tmp directory in which the sources are opened
        self.created = 1
        try:
            os.mkdir('tmp')
        except:
            self.created = 0
        #print "The folder tmp already exist, no need to create it"

        # Go to the tmp directory
        os.chdir('tmp/')
        
        # Go into the tmp folder and untar the source
        cmd = "tar -xvf " + self.path + "/" + self.source + " >/dev/null"
        #print cmd
        (os.system(cmd)) #or (sys.exit("The source can not be found "+source))
        os.chdir('..')

    def closeSource(self):
        '''
        Remove the tmp directory and the folder of the tarball if the 
        tmp directory has been created else just remove the folder of
        the tarball
        '''
        #print os.getcwd()
        cmd = "./tmp/"+self.name
        shutil.rmtree(cmd)
        if self.created == 1:
            try:
                os.rmdir('tmp')
            except:
                sys.exit('tmp directory can not be removed')
    
    def setDescription(self):
        ''' Create the object description and fill the attributes '''
        description = Description()
        description.setDescriptionFile(self)
        description.setVersion()
        description.setLicense()
        description.setDescription()
        description.setSummary()
        description.setRequires()
        self.description = description
    
    def getArch(self):
        ''' Determine is the package is of the type Noarch or Arch '''
        # Save the top directory
        original_directory = os.getcwd()

        # Change directory and look for files
        os.chdir("./tmp/"+self.name)
        # Look for the files and put them into a tmp file
        cmd = 'find ./ -name "*.c" -o -name "*.C" -o -name "*.cp" -o -name "*.cpp" -o -name "*.f"  -o -name "*.F" > tmp'
        os.system(cmd)
        # Get the size of the generated file
        size = os.path.getsize('tmp')
        # If the size of this file is 0 the package should probably be noarch
        if size == 0:
            self.arch = 'noarch'
        # Remove this file
        os.system('rm tmp')
        
        # Comes back to the original directory
        os.chdir(original_directory)
        
    def setPackager(self, bioc):
        ''' Creates the object packager and fill the attributes '''
        packager = Packager()
        packager.readConfFile(bioc)
        self.packager = packager

    def setBiocUrl(self):
        ''' Set the url of a standard Bioconductor package if the option has been used '''
        if self.source0 == None:
            if self.packager.bioc:
                urlBioc = "http://bioconductor.org/packages/%{BioC}/bioc/src/contrib/%{packname}_%{version}.tar.gz"
                self.source0 = urlBioc
        if self.packager.bioc:
            self.URL = "http://bioconductor.org/packages/%{BioC}/bioc/html/" + self.name + ".html"


    def setCranUrl(self):
        ''' Set the url of a standard Cran package if the option has been used '''
        if self.source0 == None:
                urlCran = "http://cran.r-project.org/src/contrib/%{packname}_%{version}.tar.gz"
                self.source0 = urlCran
        self.URL = "http://cran.r-project.org/web/packages/" + self.name + "/index.html"


    def writeSpec(self):
        ''' Check if the spec file already exist and generate it '''
    
        # Do something if the spec file already exist
        specname = "R-" + self.name + ".spec"
        if specname in os.listdir('./'):
            text =  "The specfile R-" + self.name + ".spec already exist !\nDo you want to overwrite it ? Y/N [N]\n"
            answer = raw_input(text)
            if answer == '':
                answer = "N"
            if answer.lower() == "n" or answer.lower() == "no":
                if specname+"-new" not in os.listdir('./'):
                    specname = specname+"-new"
                elif specname+"-new1" not in os.listdir('./'):
                    specname = specname+"-new1"
                else:
                    text = "There are already several version of the specfile in this directory, please indiquate the name you would like to give:\n"
                    specname = raw_input(text)
        
        if self.arch == "noarch":
            spec = Noarch(self)
        else:
            spec= Spec(self)
        spec.finishName()
        spec.writeSpec(specname)
        spec.writeOut()


    def moveSource(self):
        f = open(os.path.expanduser("~")+"/.rpmmacros", "r")
        rpm = f.read()
        f.close()
        try:
            topdir = re.compile('%_topdir(.*)').findall(rpm)[0].strip()
            topdir = topdir.replace('%(echo $HOME)', os.path.expanduser('~'))
            topdir = topdir+"/SOURCES/"
        except:
            topdir = None
        text = "Do you want to move the source (" + self.source + ") in " + topdir + "? Y/N [Y]\n"
        answer = raw_input(text)
        if answer == '':
            answer = "y"
        if answer.lower() == "y" or answer.lower() == "yes":
            print "cp ./" + self.source, "to", topdir
            #print os.cwd()
            try:
                shutil.copy(self.source, topdir)
            except:
                print "Can not copy the source to ",topdir
        else:
            print "Done"
