#!/usr/bin/python
#-*- coding: UTF-8 -*-

#***********************************************
# R2spec 
#
# R2spec is made to help to the creation of R specfile
# It works from a source file (*.tar.gz) or a url (http://...*.tar.gz)
#
#
# Made the 13th February 2008
# by Pierre-Yves chibon
#
#
# Distributed under License GPLv3 or later
# You can find a copy of this license on the website
# http://www.gnu.org/licenses/gpl.html
#
# This software has been based on the guidelines for R packaging
# http://fedoraproject.org/wiki/Packaging/R
#
#***********************************************

from Package import *
from optparse import OptionParser
import getopt, sys, re


USAGE ="""
This program is designed to create specfile from a source file or a url.
Usage:  R2spec -u <url> 
        R2spec -s <source>
        R2spec -s folder/folder/<source>

You can also use the following options:
  --bioc                For Bioconductor libraries
  --cran                For libraries coming from the CRAN repository

The source can be in the same directory or in a subdirectory.

The information about the packager should be filled in /etc/R2spec.conf .
"""

 
#################################################
### argument
# retrieve the arguments given and show the help when needed
def getArgument():
    ''' Handle the parameters'''
    parser = OptionParser(version="%prog 2.5.1")
    parser.add_option("-u", "--url", dest="url",
                  help="Url of the R library")
    parser.add_option("-s", "--source", dest="source",
                  help="The source file (tarball) of the R library")
    parser.add_option("--bioc", dest="bioc", default=False, action="store_true",
                  help="For Bioconductor libraries")
    parser.add_option("--cran", dest="cran", default=False, action="store_true",
                  help="For libraries coming from the CRAN repository")
    (options, args) = parser.parse_args()

    return (options.source, options.url, options.bioc, options.cran)
################################################


#################################################
### main
# Do the job :)

# Retrieve the given arguments
(source, url, bioc, cran) = getArgument()

# Return the usage message if no arguments are given
if(url == None and source == None):
    sys.exit(USAGE)

# Return an error message if there is 2 arguments given
if (url != None and source != None):
    sys.exit("\n***  Please give only one argument  ***\n")

# Fails if a library comes from both bioconductor and cran
if (cran == True and bioc == True):
    sys.exit("\n***  The library can not be --bioc and --cran at the same time  ***\n")

# Check wether the url or the source exist and if so check wether they are http:// and tar.gz
if(url != None):
    # Check wether the url start with http://
    position = url.find('http://')
    if position != 0:
        sys.exit("\nYour URL does not start with http:// => " + url + "\n")

    # Check wether the url refers to a .tar.gz
    p = re.compile('http://.*tar.gz')
    url2 = p.findall(url)
    if url2 == []:
        sys.exit("Your URL does not link to a tarball file "+url)

if(source != None):
    p = re.compile('.*\.tar\.gz')
    source = p.findall(source)[0]

package = Package()
package.setSource(url, source)
package.download()
package.setPackager(bioc)
print "\n","*"*50
print "* Name of the packager =", package.packager.name
print "* Mail of the packager =", package.packager.email
if bioc: print "* Release of Bioconductor =", package.packager.bioc
package.setName()
print "* Name of the library =", package.name
if bioc: package.setBiocUrl()
if cran: package.setCranUrl()
package.openSource()
package.setDescription()
print "* Version of the library =", package.description.version
print "* Summary of the library = ", package.description.summary
print "* Description of the library =\n   ", package.description.description
print "* License of the library =", package.description.license
package.getArch()
if package.arch == 'noarch':
    print "* The package is = Noarch" 
else:
    print "* The package is = Arch ~ x86 or x86_64" 
package.closeSource()
package.setDate()
print "* Date of the spec file =", package.date
print "*"*50,"\n\n Generate the spec file\n"
package.writeSpec()
package.moveSource()